from __future__ import annotations

import pathlib

from autpy import git


def _repo() -> git.Repo:
    return git.Repo.submissions()


def get_project_files(
    branch: str, *, exclude_grading: bool = False
) -> set[pathlib.PurePath]:
    """Get a set of all project files in the given branch."""
    paths = set()
    for path in _repo().ls_files(branch):
        if "project" not in path.parts:
            continue
        if exclude_grading and _is_grading_path(path):
            continue
        paths.add(path)
    return paths


def has_project(branch: str) -> bool:
    """Check if the given branch has project files."""
    return any(get_project_files(branch))


def _is_grading_path(path: pathlib.PurePath) -> bool:
    """Whether the given path is a grading file."""
    return path.suffix == ".md" and path.stem.split("-")[0] == "grading"


def find_grading_path(paths: list[pathlib.PurePath]) -> pathlib.PurePath | None:
    """Fin the grading file in a list of paths."""
    for p in paths:
        if _is_grading_path(p):
            return p
        assert "grading" not in p.name, p
    return None
