"""Various utility functions.

For utilities split into lots of functions (e.g. git),
consider starting a separate file!
"""

import datetime
import hashlib
from typing import cast

import humanhash  # type: ignore[import]


def semester_suffix(dt: datetime.datetime | None = None) -> str:
    if dt is None:
        dt = datetime.datetime.now()

    if 1 <= dt.month <= 2:
        year = dt.year - 1
    elif 3 <= dt.month <= 7:
        raise ValueError("Not a teaching semester right now?")
    elif 8 <= dt.month <= 12:
        year = dt.year
    else:
        raise ValueError("Broken universe or something")  # pragma: no cover

    shortyear = str(year)[2:]
    return f"hs{shortyear}"


# https://gitlab.ost.ch/ins-courses/autpy/development/-/issues/39
# FIXME remove in 2023!
NODEID_HASH_OVERRIDES = {
    "steak_wisconsin_table_india": "lion_eight_bacon_floor",
    "delaware_river_carpet_magazine": "seven_zulu_bacon_pizza",
    "louisiana_virginia_blossom_social": "failed_ceiling_triple_spring",
    "moon_fish_beer_asparagus": "fifteen_six_failed_triple",
    "video_hotel_network_mango": "carbon_finch_kitten_september",
    "sixteen_nitrogen_paris_eight": "lake_oklahoma_oregon_high",
    "wolfram_kentucky_low_chicken": "florida_snake_failed_october",
    "magnesium_nuts_winner_whiskey": "snake_louisiana_ceiling_september",
    "pennsylvania_south_solar_aspen": "august_venus_fifteen_lamp",
    "massachusetts_vermont_river_apart": "triple_johnny_romeo_ink",
    "sink_earth_timing_nuts": "jig_artist_snake_potato",
    "carpet_undress_hydrogen_burger": "charlie_two_blossom_potato",
    "lithium_north_rugby_undress": "finch_tennessee_nitrogen_sodium",
    "bakerloo_robert_november_nevada": "steak_gee_potato_bacon",
    "five_item_triple_cat": "snake_bulldog_eight_utah",
    "ohio_saturn_comet_coffee": "one_mike_alabama_venus",
    "earth_black_blossom_double": "grey_leopard_fifteen_fanta",
    "red_timing_pip_muppet": "single_mountain_hotel_vermont",
    "ten_chicken_low_march": "robert_colorado_august_sweet",
    "mexico_leopard_earth_delta": "quebec_carbon_apart_fanta",
    "pizza_foxtrot_oklahoma_network": "jersey_cat_whiskey_pluto",
    "queen_lamp_bakerloo_sixteen": "fillet_north_skylark_green",
    "illinois_north_mango_mirror": "robin_texas_spaghetti_bacon",
    "ceiling_wisconsin_butter_nebraska": "white_hot_magazine_delaware",
    "oscar_magazine_maryland_iowa": "stream_hamper_kansas_shade",
    "quebec_uniform_rugby_timing": "mars_oscar_video_winter",
    "august_romeo_pip_friend": "angel_enemy_whiskey_green",
    "zebra_ack_saturn_salami": "california_virginia_pluto_kentucky",
    "mobile_fourteen_oranges_johnny": "mango_island_speaker_juliet",
    "william_robert_ink_jig": "carpet_lima_cup_bakerloo",
    "dakota_sink_moon_nebraska": "green_carolina_high_skylark",
    "robert_bravo_pizza_artist": "twelve_carbon_oklahoma_quiet",
    "two_high_delaware_oxygen": "alabama_stream_north_seven",
    "lemon_floor_summer_comet": "papa_batman_rugby_pizza",
    "princess_pizza_early_freddie": "angel_black_burger_moon",
    "kansas_undress_four_kentucky": "robert_beryllium_finch_pip",
    "leopard_mobile_magnesium_social": "tennessee_jig_freddie_happy",
    "diet_sink_mike_fruit": "august_crazy_oklahoma_uniform",
    "paris_island_zebra_alpha": "dakota_bravo_robert_magnesium",
    "skylark_sink_lima_idaho": "three_eight_mars_emma",
    "thirteen_massachusetts_mississippi_connecticut": "vermont_muppet_salami_oranges",
    "blue_freddie_cup_skylark": "social_pizza_twelve_louisiana",
    "equal_louisiana_purple_aspen": "may_stream_social_beer",
    "seventeen_lamp_bulldog_xray": "venus_sixteen_uncle_uniform",
    "idaho_nine_washington_twelve": "texas_magazine_purple_golf",
    "beer_spaghetti_ceiling_network": "muppet_cardinal_high_lake",
    "low_massachusetts_nitrogen_mango": "georgia_berlin_paris_hotel",
    "twenty_sixteen_friend_maine": "five_steak_item_three",
    "stream_louisiana_orange_happy": "quiet_idaho_football_quiet",
    "beer_robin_spring_november": "connecticut_mountain_hydrogen_bacon",
    "maine_apart_earth_pip": "emma_maine_timing_diet",
    "sink_pip_beryllium_hamper": "london_arizona_item_romeo",
    "arkansas_table_shade_juliet": "ten_maine_lion_iowa",
    "six_hotel_chicken_orange": "lemon_seven_early_cola",
    "april_burger_mars_bulldog": "tennessee_xray_massachusetts_arkansas",
    "bluebird_cat_sad_low": "oxygen_eighteen_berlin_tango",
    "april_low_bravo_harry": "saturn_william_timing_bravo",
    "two_lamp_lion_vegan": "butter_kilo_spaghetti_edward",
    "foxtrot_glucose_eight_item": "floor_fruit_uniform_quebec",
    "cup_nine_mirror_two": "florida_fillet_zulu_blossom",
    "oscar_jersey_ohio_lamp": "robin_november_orange_happy",
    "single_lithium_west_undress": "summer_asparagus_seventeen_nineteen",
    "venus_romeo_friend_bluebird": "override_for_tests",
}


def nodeid_hash(nodeid: str) -> str:
    sha1 = hashlib.sha1(nodeid.encode("utf-8")).hexdigest()
    name = cast(str, humanhash.humanize(sha1)).replace("-", "_")
    return NODEID_HASH_OVERRIDES.get(name, name)
