from __future__ import annotations

import datetime

import pytest

from autpy import utils


@pytest.mark.parametrize(
    "dt, semester",
    [
        (datetime.datetime(2022, 1, 1), "hs21"),
        (datetime.datetime(2022, 2, 1), "hs21"),
        (datetime.datetime(2022, 3, 1), ValueError),
        (datetime.datetime(2022, 4, 1), ValueError),
        (datetime.datetime(2022, 5, 1), ValueError),
        (datetime.datetime(2022, 6, 1), ValueError),
        (datetime.datetime(2022, 7, 1), ValueError),
        (datetime.datetime(2022, 8, 1), "hs22"),
        (datetime.datetime(2022, 9, 1), "hs22"),
        (datetime.datetime(2022, 10, 1), "hs22"),
        (datetime.datetime(2022, 11, 1), "hs22"),
        (datetime.datetime(2022, 12, 1), "hs22"),
    ],
)
def test_semester_suffix(
    dt: datetime.datetime, semester: str | type[ValueError]
) -> None:
    if semester is ValueError:
        with pytest.raises(ValueError):
            utils.semester_suffix(dt)
    else:
        assert utils.semester_suffix(dt) == semester


@pytest.mark.parametrize(
    "inp, out",
    [
        ("test_spam", "sad_december_social_king"),
        ("test_override", "override_for_tests"),
    ],
)
def test_nodeid_hash(inp: str, out: str) -> None:
    assert utils.nodeid_hash(inp) == out
